import os
import re
from typing import Optional
from typing import Union

import pandas as pd

from dfiber.core.aqua import _Aqua
from dfiber.core.client.cosmos_client import CosmosClient
from dfiber.core.dfiber_exceptions import DfiberValueError, DfiberTypeError
from dfiber.core.models.constants import AuthMethod
from dfiber.core.models.credentials import Credentials


class Dfiber:
    """
    This class allows the user to connect to Dview's Aqua services and execute SQL queries.

    Attributes:
        credentials (Credentials): Dview credentials.
        catalog (str): Name of the catalog to connect to.
        schema (str): Name of the schema to connect to.
        dfiber_endpoint (str): Dfiber endpoint URL.
        cosmos_endpoint (str): Cosmos endpoint URL.
    """
    def __init__(self, credentials: Union[dict, Credentials], catalog: str, schema: Optional[str] = None, dfiber_endpoint: Optional[str] = None, cosmos_endpoint: Optional[str] = None):
        """
        Initializes a Dfiber instance.

        Args:
            credentials (Union[dict, Credentials]): Dview credentials.
            catalog (str): Name of the catalog to connect to.
            schema (str, optional): Name of the schema to connect to. Defaults to None.
            dfiber_endpoint (str, optional): Dfiber endpoint URL. Defaults to None.
            cosmos_endpoint (str, optional): Cosmos endpoint URL. Defaults to None.

        Raises:
            DfiberTypeError: If credentials are not a dictionary or an instance of the Credentials class.
            DfiberValueError: If DFIBER_ENDPOINT environment variable is not set.
        """
        if isinstance(credentials, dict):
            self._credentials = Credentials(**credentials)
        elif isinstance(credentials, Credentials):
            self._credentials = credentials
        else:
            raise DfiberTypeError(error_type="TYPE_ERROR",
                                  name="CREDENTIALS_TYPE_MISMATCH",
                                  detail="Credentials must be either a dictionary or an instance of the Credentials class.")
        self._cosmos_client = CosmosClient(cosmos_endpoint)
        self._catalog = catalog
        self._schema = schema
        if not dfiber_endpoint:
            dfiber_endpoint = os.getenv("DFIBER_ENDPOINT")
            if not dfiber_endpoint:
                raise DfiberValueError(error_type='CONFIGURATION_ERROR',
                                       name='MISSING_ENDPOINT',
                                       detail="Either the DFIBER_ENDPOINT environment variable must be set, or 'dfiber_endpoint' must be passed during initialization."
                                       )
        self._connection_args = self.parse_dfiber_endpoint(dfiber_endpoint)

    def execute_query(self, query: str):
        """
        Authenticates the user, parses connection arguments, and executes the query.

        Args:
            query (str): The SQL query to execute.

        Returns:
            pd.DataFrame: A pandas DataFrame containing the query results from Dfiber.

        """

        user_email = self.authenticate_user(self._credentials)
        self._connection_args["user"] = user_email
        aqua = _Aqua(self._connection_args)
        return aqua.execute_query(query)

    def parse_dfiber_endpoint(self, endpoint: str):
        """
        Parses an DFIBER_ENDPOINT URL and extracts the components.

        Args:
            endpoint (str): The DFIBER_ENDPOINT URL in the format
                            "dfiber://<host>:<port>/<catalog>/<schema>".

        Returns:
            dict: A dictionary with keys 'host', 'port', 'catalog', and 'schema'.
                  Missing components will be returned as `None` or default values.

        Raises:
            DfiberValueError: If the DFIBER_ENDPOINT is in an invalid format or missing required components.
        """
        pattern = r'^dfiber://(?P<host>[^:/]*)(:(?P<port>\d+))?(?P<catalog>/[^/]+)?(?P<schema>/[^/]+)?/?$'
        match = re.match(pattern, endpoint)
        if match:
            parsed_data = match.groupdict()
            if not parsed_data.get('host', None):
                raise DfiberValueError(error_type='VALUE_ERROR', name='MISSING_HOST', detail="Missing or invalid host in DFIBER_ENDPOINT")
            connection_args = {
                "host": parsed_data['host'],
                "port": int(parsed_data['port']) if parsed_data.get('port') else 8080,
                "catalog": self._catalog if self._catalog else parsed_data['catalog'].strip('/') if parsed_data.get('catalog') else None
            }
            parsed_data['schema'] = self._schema if self._schema else parsed_data['schema'].strip('/') if parsed_data.get('schema') else None
            if parsed_data.get('schema', None):
                connection_args['schema'] = parsed_data['schema']
            return connection_args
        else:
            raise DfiberValueError(error_type='VALUE_ERROR', name='INVALID_FORMAT', detail="Parse Error, Invalid DFIBER_ENDPOINT format")

    def authenticate_user(self, credentials: Credentials):
        """
        Authenticates the user and retrieves the user email and password from the provided credentials.

        Args:
            credentials (Credentials): The Dfiber credentials.

        Returns:
            string: user_email.
        """
        if credentials.auth_method == AuthMethod.EMAIL_PASSWORD:
            return self._cosmos_client.authenticate_with_email_password(credentials.email, credentials.password)
        elif credentials.auth_method == AuthMethod.API_KEY:
            return self._cosmos_client.authenticate_with_api_key(credentials.api_key)
        elif credentials.auth_method == AuthMethod.COOKIE:
            cookies = {"token": credentials.cookie}
            return self._cosmos_client.authenticate_with_cookie(cookies)

