from typing import Optional

from pydantic import BaseModel, model_validator, Field

from dfiber.core.dfiber_exceptions import DfiberValueError
from dfiber.core.models.constants import AuthMethod


class Credentials(BaseModel):
    """
    Dview credentials model.

    Attributes:
        email (Optional[str]): User email for email-password-based authentication.
        password (Optional[str]): Password for email-password-based authentication.
        api_key (Optional[str]): API key for API-key-based authentication.
        cookie (Optional[str]): Cookie for cookie-based authentication.
        auth_method (Optional[AuthMethod]): Authentication method (e.g., email-password-based, API-key-based, cookie-based).

    Raises:
         DfiberValueError: If credentials are missing or invalid.
    """
    email: Optional[str] = Field(default=None,
                                 description="User email for email-password-based authentication.")
    password: Optional[str] = Field(default=None,
                                    description="Password for email-password-based authentication.")
    cookie: Optional[str] = Field(default=None,
                                  description="Cookie for cookie-based authentication.")
    api_key: Optional[str] = Field(default=None,
                                   description="API key for API-key-based authentication.")
    auth_method: Optional[AuthMethod] = Field(default=None,
                                              description="Authentication method to be used(e.g., email-password-based, api-key-based, cookie-based).")

    @model_validator(mode='after')
    def validate_credentials(cls, values):
        email = values.email
        password = values.password
        cookie = values.cookie
        api_key = values.api_key
        auth_method = values.auth_method

        if not auth_method:
            if email and password:
                auth_method = AuthMethod.EMAIL_PASSWORD
            elif api_key:
                auth_method = AuthMethod.API_KEY
            elif cookie:
                auth_method = AuthMethod.COOKIE
            else:
                raise DfiberValueError(error_type='VALUE ERROR', name='MISSING CREDENTIALS', detail="At least one of 'email', 'password', 'api_key', or 'cookie' must be provided.")

        if auth_method == AuthMethod.EMAIL_PASSWORD:
            if not email or not password:
                raise DfiberValueError(error_type='VALUE ERROR', name='MISSING CREDENTIALS', detail="'email' and 'password' are required for email-password-based authentication.")
        elif auth_method == AuthMethod.API_KEY:
            if not api_key:
                raise DfiberValueError(error_type='VALUE ERROR', name='MISSING CREDENTIALS', detail="'api_key' is required for API-key-based authentication.")
        elif auth_method == AuthMethod.COOKIE:
            if not cookie:
                raise DfiberValueError(error_type='VALUE ERROR', name='MISSING CREDENTIALS', detail="'cookie' is required for cookie-based authentication.")
        return values
